'use strict';

var dynamic = require('@vanilla-extract/dynamic');

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"];

  if (_i == null) return;
  var _arr = [];
  var _n = true;
  var _d = false;

  var _s, _e;

  try {
    for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

var VALUE_REGEX = /(-)?\$(\w*)/;
function trim$(rawValue) {
  if (typeof rawValue === 'string') {
    var matches = rawValue.match(VALUE_REGEX);

    if (matches) {
      var _matches$;

      return ((_matches$ = matches[1]) !== null && _matches$ !== void 0 ? _matches$ : '').concat(matches[2]);
    }
  }

  return rawValue;
}
function mapValues(obj, callback) {
  var result = {};

  for (var _key in obj) {
    Object.assign(result, _defineProperty({}, _key, callback(obj[_key], _key, obj)));
  }

  return result;
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) return _arrayLikeToArray(arr);
}

function _iterableToArray(iter) {
  if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter);
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
}

function _createForOfIteratorHelper(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];

  if (!it) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;

      var F = function () {};

      return {
        s: F,
        n: function () {
          if (i >= o.length) return {
            done: true
          };
          return {
            done: false,
            value: o[i++]
          };
        },
        e: function (e) {
          throw e;
        },
        f: F
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  var normalCompletion = true,
      didErr = false,
      err;
  return {
    s: function () {
      it = it.call(o);
    },
    n: function () {
      var step = it.next();
      normalCompletion = step.done;
      return step;
    },
    e: function (e) {
      didErr = true;
      err = e;
    },
    f: function () {
      try {
        if (!normalCompletion && it.return != null) it.return();
      } finally {
        if (didErr) throw err;
      }
    }
  };
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    enumerableOnly && (symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    })), keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = null != arguments[i] ? arguments[i] : {};
    i % 2 ? ownKeys(Object(source), !0).forEach(function (key) {
      _defineProperty(target, key, source[key]);
    }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) {
      Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
    });
  }

  return target;
}

function assignClasses(propertyConfig, propValue) {
  if (!propValue && propValue !== 0) {
    return '';
  }

  var dynamic = propertyConfig.dynamic,
      values = propertyConfig.values,
      propName = propertyConfig.name; // Value is a string or number, ie not responsive

  if (typeof propValue === 'string' || typeof propValue === 'number') {
    var value = trim$(propValue); // Check for static value first

    if (values !== null && values !== void 0 && values[value]) {
      return values[value]["default"];
    }

    if (dynamic) {
      return dynamic["default"];
    } // If the property is not dynamic, and unrecognized value is provided
    // Quietly warn
    // eslint-disable-next-line no-console


    console.error("Rainbow Sprinkles: invalid value provided to prop '".concat(propName, "'. Expected one of ").concat(Object.keys(values).map(function (className) {
      return "\"".concat(className, "\"");
    }).join(', '), ". Received: ").concat(JSON.stringify(propValue), "."));
    return '';
  }

  var keys = Object.keys(propValue); // If no entries, exit gracefully

  if (keys.length < 1) {
    return '';
  }

  var className = keys.map(function (bp) {
    var rawValueAtBp = propValue[bp];
    var valueAtBp = trim$(rawValueAtBp); // Check for static value first

    if (values !== null && values !== void 0 && values[valueAtBp]) {
      return values[valueAtBp].conditions[bp];
    }

    if (dynamic) {
      return dynamic.conditions[bp];
    } // If the property is not dynamic, and unrecognized value is provided
    // Quietly warn
    // eslint-disable-next-line no-console


    console.error("Rainbow Sprinkles: invalid value provided to prop '".concat(propName, "'. Expected one of ").concat(Object.keys(values).map(function (className) {
      return "\"".concat(className, "\"");
    }).join(', '), ". Received: ").concat(JSON.stringify(valueAtBp), "."));
    return null;
  }).filter(Boolean);
  return className.join(' ').trim();
}

function _assignInlineVars(propertyConfig, propValue) {
  var vars = propertyConfig.vars,
      dynamicScale = propertyConfig.dynamicScale,
      values = propertyConfig.values,
      dynamic$1 = propertyConfig.dynamic; // Value is a string, ie not responsive

  if (typeof propValue === 'string' || typeof propValue === 'number') {
    var _values$conditions, _dynamicScale$parsedV;

    var parsedValue = trim$(propValue); // If the propValue matches a static value,
    // don't assign any variables

    if (values !== null && values !== void 0 && values[parsedValue] || values !== null && values !== void 0 && (_values$conditions = values.conditions) !== null && _values$conditions !== void 0 && _values$conditions[parsedValue] || !dynamic$1) {
      return {};
    }

    return dynamic.assignInlineVars(_defineProperty({}, vars["default"], (_dynamicScale$parsedV = dynamicScale === null || dynamicScale === void 0 ? void 0 : dynamicScale[parsedValue]) !== null && _dynamicScale$parsedV !== void 0 ? _dynamicScale$parsedV : "".concat(propValue)));
  } // If no entries, exit gracefully


  if (propValue && Object.keys(propValue).length < 1 || propValue == null) {
    return {};
  }

  var hasProperty = false;
  var variableAssignments = Object.entries(propValue).reduce(function (acc, _ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        bp = _ref2[0],
        value = _ref2[1];

    if (value) {
      var _dynamicScale$_parsed;

      var _parsedValue = trim$(value);

      if (values !== null && values !== void 0 && values[_parsedValue] || !dynamic$1) {
        // If value has a static class, don't assign any variables
        return acc;
      }

      hasProperty = true;
      acc[vars.conditions[bp]] = (_dynamicScale$_parsed = dynamicScale === null || dynamicScale === void 0 ? void 0 : dynamicScale[_parsedValue]) !== null && _dynamicScale$_parsed !== void 0 ? _dynamicScale$_parsed : "".concat(value);
    }

    return acc;
  }, {});
  return hasProperty ? dynamic.assignInlineVars(variableAssignments) : {};
}

var createRuntimeFn = function createRuntimeFn() {
  for (var _len = arguments.length, configs = new Array(_len), _key = 0; _key < _len; _key++) {
    configs[_key] = arguments[_key];
  }

  var cssConfig = Object.assign.apply(Object, [{}].concat(_toConsumableArray(configs.map(function (c) {
    return c.config;
  }))));
  var properties = Object.keys(cssConfig);
  var propertiesSet = new Set(properties);
  var shorthandNames = properties.filter(function (property) {
    return 'mappings' in cssConfig[property];
  });

  var fn = function fn(props) {
    var style = {};
    var className = [];
    var otherProps = {};
    var shorthands = {};

    var nonShorthands = _objectSpread2({}, props);

    var hasShorthands = false;

    var _iterator = _createForOfIteratorHelper(shorthandNames),
        _step;

    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var shorthand = _step.value;
        var value = props[shorthand];

        if (value != null) {
          var sprinkle = cssConfig[shorthand];
          hasShorthands = true;

          var _iterator2 = _createForOfIteratorHelper(sprinkle.mappings),
              _step2;

          try {
            for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
              var propMapping = _step2.value;
              shorthands[propMapping] = value;

              if (nonShorthands[propMapping] == null) {
                delete nonShorthands[propMapping];
              }
            }
          } catch (err) {
            _iterator2.e(err);
          } finally {
            _iterator2.f();
          }
        }
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }

    var finalProps = hasShorthands ? _objectSpread2(_objectSpread2({}, shorthands), nonShorthands) : props;

    for (var property in finalProps) {
      if (!propertiesSet.has(property)) {
        otherProps[property] = props[property];
        continue;
      }

      var propertyConfig = cssConfig[property];
      var propValue = finalProps[property];

      if ('mappings' in propertyConfig) {
        continue;
      }

      if (propertyConfig) {
        className.push(assignClasses(propertyConfig, propValue));
        Object.assign(style, _assignInlineVars(propertyConfig, propValue));
      }
    }

    return {
      className: className.join(' ').trim(),
      style: style,
      otherProps: otherProps
    };
  };

  return Object.assign(fn, {
    properties: propertiesSet
  });
};

exports._defineProperty = _defineProperty;
exports._slicedToArray = _slicedToArray;
exports._toConsumableArray = _toConsumableArray;
exports.createRuntimeFn = createRuntimeFn;
exports.mapValues = mapValues;
